/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package model

import (
	"context"
	"fmt"
	"github.com/apache/plc4x/plc4go/spi/utils"
	"github.com/pkg/errors"
	"github.com/rs/zerolog"
)

// Code generated by code-generation. DO NOT EDIT.

// BACnetApplicationTagNull is the corresponding interface of BACnetApplicationTagNull
type BACnetApplicationTagNull interface {
	fmt.Stringer
	utils.LengthAware
	utils.Serializable
	BACnetApplicationTag
}

// BACnetApplicationTagNullExactly can be used when we want exactly this type and not a type which fulfills BACnetApplicationTagNull.
// This is useful for switch cases.
type BACnetApplicationTagNullExactly interface {
	BACnetApplicationTagNull
	isBACnetApplicationTagNull() bool
}

// _BACnetApplicationTagNull is the data-structure of this message
type _BACnetApplicationTagNull struct {
	*_BACnetApplicationTag
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/////////////////////// Accessors for discriminator values.
///////////////////////

///////////////////////
///////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

func (m *_BACnetApplicationTagNull) InitializeParent(parent BACnetApplicationTag, header BACnetTagHeader) {
	m.Header = header
}

func (m *_BACnetApplicationTagNull) GetParent() BACnetApplicationTag {
	return m._BACnetApplicationTag
}

// NewBACnetApplicationTagNull factory function for _BACnetApplicationTagNull
func NewBACnetApplicationTagNull(header BACnetTagHeader) *_BACnetApplicationTagNull {
	_result := &_BACnetApplicationTagNull{
		_BACnetApplicationTag: NewBACnetApplicationTag(header),
	}
	_result._BACnetApplicationTag._BACnetApplicationTagChildRequirements = _result
	return _result
}

// Deprecated: use the interface for direct cast
func CastBACnetApplicationTagNull(structType any) BACnetApplicationTagNull {
	if casted, ok := structType.(BACnetApplicationTagNull); ok {
		return casted
	}
	if casted, ok := structType.(*BACnetApplicationTagNull); ok {
		return *casted
	}
	return nil
}

func (m *_BACnetApplicationTagNull) GetTypeName() string {
	return "BACnetApplicationTagNull"
}

func (m *_BACnetApplicationTagNull) GetLengthInBits(ctx context.Context) uint16 {
	lengthInBits := uint16(m.GetParentLengthInBits(ctx))

	return lengthInBits
}

func (m *_BACnetApplicationTagNull) GetLengthInBytes(ctx context.Context) uint16 {
	return m.GetLengthInBits(ctx) / 8
}

func BACnetApplicationTagNullParse(ctx context.Context, theBytes []byte) (BACnetApplicationTagNull, error) {
	return BACnetApplicationTagNullParseWithBuffer(ctx, utils.NewReadBufferByteBased(theBytes))
}

func BACnetApplicationTagNullParseWithBuffer(ctx context.Context, readBuffer utils.ReadBuffer) (BACnetApplicationTagNull, error) {
	positionAware := readBuffer
	_ = positionAware
	log := zerolog.Ctx(ctx)
	_ = log
	if pullErr := readBuffer.PullContext("BACnetApplicationTagNull"); pullErr != nil {
		return nil, errors.Wrap(pullErr, "Error pulling for BACnetApplicationTagNull")
	}
	currentPos := positionAware.GetPos()
	_ = currentPos

	if closeErr := readBuffer.CloseContext("BACnetApplicationTagNull"); closeErr != nil {
		return nil, errors.Wrap(closeErr, "Error closing for BACnetApplicationTagNull")
	}

	// Create a partially initialized instance
	_child := &_BACnetApplicationTagNull{
		_BACnetApplicationTag: &_BACnetApplicationTag{},
	}
	_child._BACnetApplicationTag._BACnetApplicationTagChildRequirements = _child
	return _child, nil
}

func (m *_BACnetApplicationTagNull) Serialize() ([]byte, error) {
	wb := utils.NewWriteBufferByteBased(utils.WithInitialSizeForByteBasedBuffer(int(m.GetLengthInBytes(context.Background()))))
	if err := m.SerializeWithWriteBuffer(context.Background(), wb); err != nil {
		return nil, err
	}
	return wb.GetBytes(), nil
}

func (m *_BACnetApplicationTagNull) SerializeWithWriteBuffer(ctx context.Context, writeBuffer utils.WriteBuffer) error {
	positionAware := writeBuffer
	_ = positionAware
	log := zerolog.Ctx(ctx)
	_ = log
	ser := func() error {
		if pushErr := writeBuffer.PushContext("BACnetApplicationTagNull"); pushErr != nil {
			return errors.Wrap(pushErr, "Error pushing for BACnetApplicationTagNull")
		}

		if popErr := writeBuffer.PopContext("BACnetApplicationTagNull"); popErr != nil {
			return errors.Wrap(popErr, "Error popping for BACnetApplicationTagNull")
		}
		return nil
	}
	return m.SerializeParent(ctx, writeBuffer, m, ser)
}

func (m *_BACnetApplicationTagNull) isBACnetApplicationTagNull() bool {
	return true
}

func (m *_BACnetApplicationTagNull) String() string {
	if m == nil {
		return "<nil>"
	}
	writeBuffer := utils.NewWriteBufferBoxBasedWithOptions(true, true)
	if err := writeBuffer.WriteSerializable(context.Background(), m); err != nil {
		return err.Error()
	}
	return writeBuffer.GetBox().String()
}
