/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package model

import (
	"context"
	"fmt"
	"github.com/apache/plc4x/plc4go/spi/utils"
	"github.com/pkg/errors"
	"github.com/rs/zerolog"
)

// Code generated by code-generation. DO NOT EDIT.

// DiscoveryConfiguration is the corresponding interface of DiscoveryConfiguration
type DiscoveryConfiguration interface {
	fmt.Stringer
	utils.LengthAware
	utils.Serializable
	ExtensionObjectDefinition
}

// DiscoveryConfigurationExactly can be used when we want exactly this type and not a type which fulfills DiscoveryConfiguration.
// This is useful for switch cases.
type DiscoveryConfigurationExactly interface {
	DiscoveryConfiguration
	isDiscoveryConfiguration() bool
}

// _DiscoveryConfiguration is the data-structure of this message
type _DiscoveryConfiguration struct {
	*_ExtensionObjectDefinition
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/////////////////////// Accessors for discriminator values.
///////////////////////

func (m *_DiscoveryConfiguration) GetIdentifier() string {
	return "12892"
}

///////////////////////
///////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

func (m *_DiscoveryConfiguration) InitializeParent(parent ExtensionObjectDefinition) {}

func (m *_DiscoveryConfiguration) GetParent() ExtensionObjectDefinition {
	return m._ExtensionObjectDefinition
}

// NewDiscoveryConfiguration factory function for _DiscoveryConfiguration
func NewDiscoveryConfiguration() *_DiscoveryConfiguration {
	_result := &_DiscoveryConfiguration{
		_ExtensionObjectDefinition: NewExtensionObjectDefinition(),
	}
	_result._ExtensionObjectDefinition._ExtensionObjectDefinitionChildRequirements = _result
	return _result
}

// Deprecated: use the interface for direct cast
func CastDiscoveryConfiguration(structType any) DiscoveryConfiguration {
	if casted, ok := structType.(DiscoveryConfiguration); ok {
		return casted
	}
	if casted, ok := structType.(*DiscoveryConfiguration); ok {
		return *casted
	}
	return nil
}

func (m *_DiscoveryConfiguration) GetTypeName() string {
	return "DiscoveryConfiguration"
}

func (m *_DiscoveryConfiguration) GetLengthInBits(ctx context.Context) uint16 {
	lengthInBits := uint16(m.GetParentLengthInBits(ctx))

	return lengthInBits
}

func (m *_DiscoveryConfiguration) GetLengthInBytes(ctx context.Context) uint16 {
	return m.GetLengthInBits(ctx) / 8
}

func DiscoveryConfigurationParse(ctx context.Context, theBytes []byte, identifier string) (DiscoveryConfiguration, error) {
	return DiscoveryConfigurationParseWithBuffer(ctx, utils.NewReadBufferByteBased(theBytes), identifier)
}

func DiscoveryConfigurationParseWithBuffer(ctx context.Context, readBuffer utils.ReadBuffer, identifier string) (DiscoveryConfiguration, error) {
	positionAware := readBuffer
	_ = positionAware
	log := zerolog.Ctx(ctx)
	_ = log
	if pullErr := readBuffer.PullContext("DiscoveryConfiguration"); pullErr != nil {
		return nil, errors.Wrap(pullErr, "Error pulling for DiscoveryConfiguration")
	}
	currentPos := positionAware.GetPos()
	_ = currentPos

	if closeErr := readBuffer.CloseContext("DiscoveryConfiguration"); closeErr != nil {
		return nil, errors.Wrap(closeErr, "Error closing for DiscoveryConfiguration")
	}

	// Create a partially initialized instance
	_child := &_DiscoveryConfiguration{
		_ExtensionObjectDefinition: &_ExtensionObjectDefinition{},
	}
	_child._ExtensionObjectDefinition._ExtensionObjectDefinitionChildRequirements = _child
	return _child, nil
}

func (m *_DiscoveryConfiguration) Serialize() ([]byte, error) {
	wb := utils.NewWriteBufferByteBased(utils.WithInitialSizeForByteBasedBuffer(int(m.GetLengthInBytes(context.Background()))))
	if err := m.SerializeWithWriteBuffer(context.Background(), wb); err != nil {
		return nil, err
	}
	return wb.GetBytes(), nil
}

func (m *_DiscoveryConfiguration) SerializeWithWriteBuffer(ctx context.Context, writeBuffer utils.WriteBuffer) error {
	positionAware := writeBuffer
	_ = positionAware
	log := zerolog.Ctx(ctx)
	_ = log
	ser := func() error {
		if pushErr := writeBuffer.PushContext("DiscoveryConfiguration"); pushErr != nil {
			return errors.Wrap(pushErr, "Error pushing for DiscoveryConfiguration")
		}

		if popErr := writeBuffer.PopContext("DiscoveryConfiguration"); popErr != nil {
			return errors.Wrap(popErr, "Error popping for DiscoveryConfiguration")
		}
		return nil
	}
	return m.SerializeParent(ctx, writeBuffer, m, ser)
}

func (m *_DiscoveryConfiguration) isDiscoveryConfiguration() bool {
	return true
}

func (m *_DiscoveryConfiguration) String() string {
	if m == nil {
		return "<nil>"
	}
	writeBuffer := utils.NewWriteBufferBoxBasedWithOptions(true, true)
	if err := writeBuffer.WriteSerializable(context.Background(), m); err != nil {
		return err.Error()
	}
	return writeBuffer.GetBox().String()
}
