/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.knxnetip.readwrite;

import static org.apache.plc4x.java.spi.codegen.fields.FieldReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.fields.FieldWriterFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataWriterFactory.*;
import static org.apache.plc4x.java.spi.generation.StaticHelper.*;

import java.time.*;
import java.util.*;
import org.apache.plc4x.java.api.exceptions.*;
import org.apache.plc4x.java.api.value.*;
import org.apache.plc4x.java.spi.codegen.*;
import org.apache.plc4x.java.spi.codegen.fields.*;
import org.apache.plc4x.java.spi.codegen.io.*;
import org.apache.plc4x.java.spi.generation.*;

// Code generated by code-generation. DO NOT EDIT.

public abstract class ServiceId implements Message {

  // Abstract accessors for discriminator values.
  public abstract Short getServiceType();

  public ServiceId() {
    super();
  }

  protected abstract void serializeServiceIdChild(WriteBuffer writeBuffer)
      throws SerializationException;

  public void serialize(WriteBuffer writeBuffer) throws SerializationException {
    PositionAware positionAware = writeBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();
    writeBuffer.pushContext("ServiceId");

    // Discriminator Field (serviceType) (Used as input to a switch field)
    writeDiscriminatorField("serviceType", getServiceType(), writeUnsignedShort(writeBuffer, 8));

    // Switch field (Serialize the sub-type)
    serializeServiceIdChild(writeBuffer);

    writeBuffer.popContext("ServiceId");
  }

  @Override
  public int getLengthInBytes() {
    return (int) Math.ceil((float) getLengthInBits() / 8.0);
  }

  @Override
  public int getLengthInBits() {
    int lengthInBits = 0;
    ServiceId _value = this;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    // Discriminator Field (serviceType)
    lengthInBits += 8;

    // Length of sub-type elements will be added by sub-type...

    return lengthInBits;
  }

  public static ServiceId staticParse(ReadBuffer readBuffer, Object... args) throws ParseException {
    PositionAware positionAware = readBuffer;
    return staticParse(readBuffer);
  }

  public static ServiceId staticParse(ReadBuffer readBuffer) throws ParseException {
    readBuffer.pullContext("ServiceId");
    PositionAware positionAware = readBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    short serviceType = readDiscriminatorField("serviceType", readUnsignedShort(readBuffer, 8));

    // Switch Field (Depending on the discriminator values, passes the instantiation to a sub-type)
    ServiceIdBuilder builder = null;
    if (EvaluationHelper.equals(serviceType, (short) 0x02)) {
      builder = KnxNetIpCore.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x03)) {
      builder = KnxNetIpDeviceManagement.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x04)) {
      builder = KnxNetIpTunneling.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x05)) {
      builder = KnxNetIpRouting.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x06)) {
      builder = KnxNetRemoteLogging.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x07)) {
      builder = KnxNetRemoteConfigurationAndDiagnosis.staticParseServiceIdBuilder(readBuffer);
    } else if (EvaluationHelper.equals(serviceType, (short) 0x08)) {
      builder = KnxNetObjectServer.staticParseServiceIdBuilder(readBuffer);
    }
    if (builder == null) {
      throw new ParseException(
          "Unsupported case for discriminated type"
              + " parameters ["
              + "serviceType="
              + serviceType
              + "]");
    }

    readBuffer.closeContext("ServiceId");
    // Create the instance
    ServiceId _serviceId = builder.build();
    return _serviceId;
  }

  public interface ServiceIdBuilder {
    ServiceId build();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof ServiceId)) {
      return false;
    }
    ServiceId that = (ServiceId) o;
    return true;
  }

  @Override
  public int hashCode() {
    return Objects.hash();
  }

  @Override
  public String toString() {
    WriteBufferBoxBased writeBufferBoxBased = new WriteBufferBoxBased(true, true);
    try {
      writeBufferBoxBased.writeSerializable(this);
    } catch (SerializationException e) {
      throw new RuntimeException(e);
    }
    return "\n" + writeBufferBoxBased.getBox().toString() + "\n";
  }
}
