/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.knxnetip.model;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.plc4x.java.knxnetip.ets.model.GroupAddress;
import org.apache.plc4x.java.knxnetip.tag.KnxNetIpTag;
import org.apache.plc4x.java.spi.messages.PlcSubscriber;
import org.apache.plc4x.java.spi.model.DefaultPlcSubscriptionHandle;

public class KnxNetIpSubscriptionHandle extends DefaultPlcSubscriptionHandle {

    private final KnxNetIpTag tag;

    public KnxNetIpSubscriptionHandle(PlcSubscriber plcSubscriber, KnxNetIpTag tag) {
        super(plcSubscriber);
        this.tag = tag;
    }

    public KnxNetIpTag getTag() {
        return tag;
    }

    public boolean matches(GroupAddress address) {
        return tag.matchesGroupAddress(address);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (!(o instanceof KnxNetIpSubscriptionHandle)) {
            return false;
        }

        KnxNetIpSubscriptionHandle that = (KnxNetIpSubscriptionHandle) o;

        return new EqualsBuilder()
            .append(getTag(), that.getTag())
            .isEquals();
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder(17, 37)
            .appendSuper(super.hashCode())
            .append(getTag())
            .toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
            .append("tag", tag)
            .toString();
    }

}
