/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.opcua.readwrite;

import static org.apache.plc4x.java.spi.codegen.fields.FieldReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.fields.FieldWriterFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataWriterFactory.*;
import static org.apache.plc4x.java.spi.generation.StaticHelper.*;

import java.time.*;
import java.util.*;
import org.apache.plc4x.java.api.exceptions.*;
import org.apache.plc4x.java.api.value.*;
import org.apache.plc4x.java.spi.codegen.*;
import org.apache.plc4x.java.spi.codegen.fields.*;
import org.apache.plc4x.java.spi.codegen.io.*;
import org.apache.plc4x.java.spi.generation.*;

// Code generated by code-generation. DO NOT EDIT.

public class ServiceCounterDataType extends ExtensionObjectDefinition implements Message {

  // Accessors for discriminator values.
  public String getIdentifier() {
    return (String) "873";
  }

  // Properties.
  protected final long totalCount;
  protected final long errorCount;

  public ServiceCounterDataType(long totalCount, long errorCount) {
    super();
    this.totalCount = totalCount;
    this.errorCount = errorCount;
  }

  public long getTotalCount() {
    return totalCount;
  }

  public long getErrorCount() {
    return errorCount;
  }

  @Override
  protected void serializeExtensionObjectDefinitionChild(WriteBuffer writeBuffer)
      throws SerializationException {
    PositionAware positionAware = writeBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();
    writeBuffer.pushContext("ServiceCounterDataType");

    // Simple Field (totalCount)
    writeSimpleField("totalCount", totalCount, writeUnsignedLong(writeBuffer, 32));

    // Simple Field (errorCount)
    writeSimpleField("errorCount", errorCount, writeUnsignedLong(writeBuffer, 32));

    writeBuffer.popContext("ServiceCounterDataType");
  }

  @Override
  public int getLengthInBytes() {
    return (int) Math.ceil((float) getLengthInBits() / 8.0);
  }

  @Override
  public int getLengthInBits() {
    int lengthInBits = super.getLengthInBits();
    ServiceCounterDataType _value = this;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    // Simple field (totalCount)
    lengthInBits += 32;

    // Simple field (errorCount)
    lengthInBits += 32;

    return lengthInBits;
  }

  public static ExtensionObjectDefinitionBuilder staticParseExtensionObjectDefinitionBuilder(
      ReadBuffer readBuffer, String identifier) throws ParseException {
    readBuffer.pullContext("ServiceCounterDataType");
    PositionAware positionAware = readBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    long totalCount = readSimpleField("totalCount", readUnsignedLong(readBuffer, 32));

    long errorCount = readSimpleField("errorCount", readUnsignedLong(readBuffer, 32));

    readBuffer.closeContext("ServiceCounterDataType");
    // Create the instance
    return new ServiceCounterDataTypeBuilderImpl(totalCount, errorCount);
  }

  public static class ServiceCounterDataTypeBuilderImpl
      implements ExtensionObjectDefinition.ExtensionObjectDefinitionBuilder {
    private final long totalCount;
    private final long errorCount;

    public ServiceCounterDataTypeBuilderImpl(long totalCount, long errorCount) {
      this.totalCount = totalCount;
      this.errorCount = errorCount;
    }

    public ServiceCounterDataType build() {
      ServiceCounterDataType serviceCounterDataType =
          new ServiceCounterDataType(totalCount, errorCount);
      return serviceCounterDataType;
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof ServiceCounterDataType)) {
      return false;
    }
    ServiceCounterDataType that = (ServiceCounterDataType) o;
    return (getTotalCount() == that.getTotalCount())
        && (getErrorCount() == that.getErrorCount())
        && super.equals(that)
        && true;
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), getTotalCount(), getErrorCount());
  }

  @Override
  public String toString() {
    WriteBufferBoxBased writeBufferBoxBased = new WriteBufferBoxBased(true, true);
    try {
      writeBufferBoxBased.writeSerializable(this);
    } catch (SerializationException e) {
      throw new RuntimeException(e);
    }
    return "\n" + writeBufferBoxBased.getBox().toString() + "\n";
  }
}
