/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.opcua.readwrite;

import static org.apache.plc4x.java.spi.codegen.fields.FieldReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.fields.FieldWriterFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataReaderFactory.*;
import static org.apache.plc4x.java.spi.codegen.io.DataWriterFactory.*;
import static org.apache.plc4x.java.spi.generation.StaticHelper.*;

import java.time.*;
import java.util.*;
import org.apache.plc4x.java.api.exceptions.*;
import org.apache.plc4x.java.api.value.*;
import org.apache.plc4x.java.spi.codegen.*;
import org.apache.plc4x.java.spi.codegen.fields.*;
import org.apache.plc4x.java.spi.codegen.io.*;
import org.apache.plc4x.java.spi.generation.*;

// Code generated by code-generation. DO NOT EDIT.

public class UserNameIdentityToken extends UserIdentityTokenDefinition implements Message {

  // Accessors for discriminator values.
  public String getIdentifier() {
    return (String) "username";
  }

  // Properties.
  protected final PascalString userName;
  protected final PascalByteString password;
  protected final PascalString encryptionAlgorithm;

  public UserNameIdentityToken(
      PascalString userName, PascalByteString password, PascalString encryptionAlgorithm) {
    super();
    this.userName = userName;
    this.password = password;
    this.encryptionAlgorithm = encryptionAlgorithm;
  }

  public PascalString getUserName() {
    return userName;
  }

  public PascalByteString getPassword() {
    return password;
  }

  public PascalString getEncryptionAlgorithm() {
    return encryptionAlgorithm;
  }

  @Override
  protected void serializeUserIdentityTokenDefinitionChild(WriteBuffer writeBuffer)
      throws SerializationException {
    PositionAware positionAware = writeBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();
    writeBuffer.pushContext("UserNameIdentityToken");

    // Simple Field (userName)
    writeSimpleField("userName", userName, new DataWriterComplexDefault<>(writeBuffer));

    // Simple Field (password)
    writeSimpleField("password", password, new DataWriterComplexDefault<>(writeBuffer));

    // Simple Field (encryptionAlgorithm)
    writeSimpleField(
        "encryptionAlgorithm", encryptionAlgorithm, new DataWriterComplexDefault<>(writeBuffer));

    writeBuffer.popContext("UserNameIdentityToken");
  }

  @Override
  public int getLengthInBytes() {
    return (int) Math.ceil((float) getLengthInBits() / 8.0);
  }

  @Override
  public int getLengthInBits() {
    int lengthInBits = super.getLengthInBits();
    UserNameIdentityToken _value = this;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    // Simple field (userName)
    lengthInBits += userName.getLengthInBits();

    // Simple field (password)
    lengthInBits += password.getLengthInBits();

    // Simple field (encryptionAlgorithm)
    lengthInBits += encryptionAlgorithm.getLengthInBits();

    return lengthInBits;
  }

  public static UserIdentityTokenDefinitionBuilder staticParseUserIdentityTokenDefinitionBuilder(
      ReadBuffer readBuffer, String identifier) throws ParseException {
    readBuffer.pullContext("UserNameIdentityToken");
    PositionAware positionAware = readBuffer;
    boolean _lastItem = ThreadLocalHelper.lastItemThreadLocal.get();

    PascalString userName =
        readSimpleField(
            "userName",
            new DataReaderComplexDefault<>(() -> PascalString.staticParse(readBuffer), readBuffer));

    PascalByteString password =
        readSimpleField(
            "password",
            new DataReaderComplexDefault<>(
                () -> PascalByteString.staticParse(readBuffer), readBuffer));

    PascalString encryptionAlgorithm =
        readSimpleField(
            "encryptionAlgorithm",
            new DataReaderComplexDefault<>(() -> PascalString.staticParse(readBuffer), readBuffer));

    readBuffer.closeContext("UserNameIdentityToken");
    // Create the instance
    return new UserNameIdentityTokenBuilderImpl(userName, password, encryptionAlgorithm);
  }

  public static class UserNameIdentityTokenBuilderImpl
      implements UserIdentityTokenDefinition.UserIdentityTokenDefinitionBuilder {
    private final PascalString userName;
    private final PascalByteString password;
    private final PascalString encryptionAlgorithm;

    public UserNameIdentityTokenBuilderImpl(
        PascalString userName, PascalByteString password, PascalString encryptionAlgorithm) {
      this.userName = userName;
      this.password = password;
      this.encryptionAlgorithm = encryptionAlgorithm;
    }

    public UserNameIdentityToken build() {
      UserNameIdentityToken userNameIdentityToken =
          new UserNameIdentityToken(userName, password, encryptionAlgorithm);
      return userNameIdentityToken;
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof UserNameIdentityToken)) {
      return false;
    }
    UserNameIdentityToken that = (UserNameIdentityToken) o;
    return (getUserName() == that.getUserName())
        && (getPassword() == that.getPassword())
        && (getEncryptionAlgorithm() == that.getEncryptionAlgorithm())
        && super.equals(that)
        && true;
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), getUserName(), getPassword(), getEncryptionAlgorithm());
  }

  @Override
  public String toString() {
    WriteBufferBoxBased writeBufferBoxBased = new WriteBufferBoxBased(true, true);
    try {
      writeBufferBoxBased.writeSerializable(this);
    } catch (SerializationException e) {
      throw new RuntimeException(e);
    }
    return "\n" + writeBufferBoxBased.getBox().toString() + "\n";
  }
}
